import { PublicKey, Connection } from "@solana/web3.js";
import BN from "bn.js";
export interface GlobalConfigFields {
    emergencyMode: BN;
    blockDeposit: BN;
    blockInvest: BN;
    blockWithdraw: BN;
    blockCollectFees: BN;
    blockCollectRewards: BN;
    blockSwapRewards: BN;
    blockSwapUnevenVaults: number;
    blockEmergencySwap: number;
    minWithdrawalFeeBps: BN;
    scopeProgramId: PublicKey;
    scopePriceId: PublicKey;
    swapRewardsDiscountBps: Array<BN>;
    actionsAuthority: PublicKey;
    adminAuthority: PublicKey;
    treasuryFeeVaults: Array<PublicKey>;
    tokenInfos: PublicKey;
    blockLocalAdmin: BN;
    minPerformanceFeeBps: BN;
    minSwapUnevenSlippageToleranceBps: BN;
    minReferencePriceSlippageToleranceBps: BN;
    actionsAfterRebalanceDelaySeconds: BN;
    treasuryFeeVaultReceiver: PublicKey;
    padding: Array<BN>;
}
export interface GlobalConfigJSON {
    emergencyMode: string;
    blockDeposit: string;
    blockInvest: string;
    blockWithdraw: string;
    blockCollectFees: string;
    blockCollectRewards: string;
    blockSwapRewards: string;
    blockSwapUnevenVaults: number;
    blockEmergencySwap: number;
    minWithdrawalFeeBps: string;
    scopeProgramId: string;
    scopePriceId: string;
    swapRewardsDiscountBps: Array<string>;
    actionsAuthority: string;
    adminAuthority: string;
    treasuryFeeVaults: Array<string>;
    tokenInfos: string;
    blockLocalAdmin: string;
    minPerformanceFeeBps: string;
    minSwapUnevenSlippageToleranceBps: string;
    minReferencePriceSlippageToleranceBps: string;
    actionsAfterRebalanceDelaySeconds: string;
    treasuryFeeVaultReceiver: string;
    padding: Array<string>;
}
export declare class GlobalConfig {
    readonly emergencyMode: BN;
    readonly blockDeposit: BN;
    readonly blockInvest: BN;
    readonly blockWithdraw: BN;
    readonly blockCollectFees: BN;
    readonly blockCollectRewards: BN;
    readonly blockSwapRewards: BN;
    readonly blockSwapUnevenVaults: number;
    readonly blockEmergencySwap: number;
    readonly minWithdrawalFeeBps: BN;
    readonly scopeProgramId: PublicKey;
    readonly scopePriceId: PublicKey;
    readonly swapRewardsDiscountBps: Array<BN>;
    readonly actionsAuthority: PublicKey;
    readonly adminAuthority: PublicKey;
    readonly treasuryFeeVaults: Array<PublicKey>;
    readonly tokenInfos: PublicKey;
    readonly blockLocalAdmin: BN;
    readonly minPerformanceFeeBps: BN;
    readonly minSwapUnevenSlippageToleranceBps: BN;
    readonly minReferencePriceSlippageToleranceBps: BN;
    readonly actionsAfterRebalanceDelaySeconds: BN;
    readonly treasuryFeeVaultReceiver: PublicKey;
    readonly padding: Array<BN>;
    static readonly discriminator: Buffer<ArrayBuffer>;
    static readonly layout: any;
    constructor(fields: GlobalConfigFields);
    static fetch(c: Connection, address: PublicKey, programId?: PublicKey): Promise<GlobalConfig | null>;
    static fetchMultiple(c: Connection, addresses: PublicKey[], programId?: PublicKey): Promise<Array<GlobalConfig | null>>;
    static decode(data: Buffer): GlobalConfig;
    toJSON(): GlobalConfigJSON;
    static fromJSON(obj: GlobalConfigJSON): GlobalConfig;
}
//# sourceMappingURL=GlobalConfig.d.ts.map