"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProtocolPositionState = void 0;
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js")); // eslint-disable-line @typescript-eslint/no-unused-vars
const borsh = __importStar(require("@coral-xyz/borsh")); // eslint-disable-line @typescript-eslint/no-unused-vars
const programId_1 = require("../programId");
class ProtocolPositionState {
    /** Bump to identify PDA */
    bump;
    /** The ID of the pool with which this token is connected */
    poolId;
    /** The lower bound tick of the position */
    tickLowerIndex;
    /** The upper bound tick of the position */
    tickUpperIndex;
    /** The amount of liquidity owned by this position */
    liquidity;
    /** The token_0 fee growth per unit of liquidity as of the last update to liquidity or fees owed */
    feeGrowthInside0LastX64;
    /** The token_1 fee growth per unit of liquidity as of the last update to liquidity or fees owed */
    feeGrowthInside1LastX64;
    /** The fees owed to the position owner in token_0 */
    tokenFeesOwed0;
    /** The fees owed to the position owner in token_1 */
    tokenFeesOwed1;
    /** The reward growth per unit of liquidity as of the last update to liquidity */
    rewardGrowthInside;
    padding;
    static discriminator = Buffer.from([
        100, 226, 145, 99, 146, 218, 160, 106,
    ]);
    static layout = borsh.struct([
        borsh.u8("bump"),
        borsh.publicKey("poolId"),
        borsh.i32("tickLowerIndex"),
        borsh.i32("tickUpperIndex"),
        borsh.u128("liquidity"),
        borsh.u128("feeGrowthInside0LastX64"),
        borsh.u128("feeGrowthInside1LastX64"),
        borsh.u64("tokenFeesOwed0"),
        borsh.u64("tokenFeesOwed1"),
        borsh.array(borsh.u128(), 3, "rewardGrowthInside"),
        borsh.array(borsh.u64(), 8, "padding"),
    ]);
    constructor(fields) {
        this.bump = fields.bump;
        this.poolId = fields.poolId;
        this.tickLowerIndex = fields.tickLowerIndex;
        this.tickUpperIndex = fields.tickUpperIndex;
        this.liquidity = fields.liquidity;
        this.feeGrowthInside0LastX64 = fields.feeGrowthInside0LastX64;
        this.feeGrowthInside1LastX64 = fields.feeGrowthInside1LastX64;
        this.tokenFeesOwed0 = fields.tokenFeesOwed0;
        this.tokenFeesOwed1 = fields.tokenFeesOwed1;
        this.rewardGrowthInside = fields.rewardGrowthInside;
        this.padding = fields.padding;
    }
    static async fetch(c, address, programId = programId_1.PROGRAM_ID) {
        const info = await c.getAccountInfo(address);
        if (info === null) {
            return null;
        }
        if (!info.owner.equals(programId)) {
            throw new Error("account doesn't belong to this program");
        }
        return this.decode(info.data);
    }
    static async fetchMultiple(c, addresses, programId = programId_1.PROGRAM_ID) {
        const infos = await c.getMultipleAccountsInfo(addresses);
        return infos.map((info) => {
            if (info === null) {
                return null;
            }
            if (!info.owner.equals(programId)) {
                throw new Error("account doesn't belong to this program");
            }
            return this.decode(info.data);
        });
    }
    static decode(data) {
        if (!data.slice(0, 8).equals(ProtocolPositionState.discriminator)) {
            throw new Error("invalid account discriminator");
        }
        const dec = ProtocolPositionState.layout.decode(data.slice(8));
        return new ProtocolPositionState({
            bump: dec.bump,
            poolId: dec.poolId,
            tickLowerIndex: dec.tickLowerIndex,
            tickUpperIndex: dec.tickUpperIndex,
            liquidity: dec.liquidity,
            feeGrowthInside0LastX64: dec.feeGrowthInside0LastX64,
            feeGrowthInside1LastX64: dec.feeGrowthInside1LastX64,
            tokenFeesOwed0: dec.tokenFeesOwed0,
            tokenFeesOwed1: dec.tokenFeesOwed1,
            rewardGrowthInside: dec.rewardGrowthInside,
            padding: dec.padding,
        });
    }
    toJSON() {
        return {
            bump: this.bump,
            poolId: this.poolId.toString(),
            tickLowerIndex: this.tickLowerIndex,
            tickUpperIndex: this.tickUpperIndex,
            liquidity: this.liquidity.toString(),
            feeGrowthInside0LastX64: this.feeGrowthInside0LastX64.toString(),
            feeGrowthInside1LastX64: this.feeGrowthInside1LastX64.toString(),
            tokenFeesOwed0: this.tokenFeesOwed0.toString(),
            tokenFeesOwed1: this.tokenFeesOwed1.toString(),
            rewardGrowthInside: this.rewardGrowthInside.map((item) => item.toString()),
            padding: this.padding.map((item) => item.toString()),
        };
    }
    static fromJSON(obj) {
        return new ProtocolPositionState({
            bump: obj.bump,
            poolId: new web3_js_1.PublicKey(obj.poolId),
            tickLowerIndex: obj.tickLowerIndex,
            tickUpperIndex: obj.tickUpperIndex,
            liquidity: new bn_js_1.default(obj.liquidity),
            feeGrowthInside0LastX64: new bn_js_1.default(obj.feeGrowthInside0LastX64),
            feeGrowthInside1LastX64: new bn_js_1.default(obj.feeGrowthInside1LastX64),
            tokenFeesOwed0: new bn_js_1.default(obj.tokenFeesOwed0),
            tokenFeesOwed1: new bn_js_1.default(obj.tokenFeesOwed1),
            rewardGrowthInside: obj.rewardGrowthInside.map((item) => new bn_js_1.default(item)),
            padding: obj.padding.map((item) => new bn_js_1.default(item)),
        });
    }
}
exports.ProtocolPositionState = ProtocolPositionState;
//# sourceMappingURL=ProtocolPositionState.js.map