"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinArray = void 0;
const web3_js_1 = require("@solana/web3.js");
const bn_js_1 = __importDefault(require("bn.js")); // eslint-disable-line @typescript-eslint/no-unused-vars
const borsh = __importStar(require("@coral-xyz/borsh")); // eslint-disable-line @typescript-eslint/no-unused-vars
const types = __importStar(require("../types")); // eslint-disable-line @typescript-eslint/no-unused-vars
const programId_1 = require("../programId");
/**
 * An account to contain a range of bin. For example: Bin 100 <-> 200.
 * For example:
 * BinArray index: 0 contains bin 0 <-> 599
 * index: 2 contains bin 600 <-> 1199, ...
 */
class BinArray {
    index;
    /** Version of binArray */
    version;
    padding;
    lbPair;
    bins;
    static discriminator = Buffer.from([
        92, 142, 92, 220, 5, 148, 70, 181,
    ]);
    static layout = borsh.struct([
        borsh.i64("index"),
        borsh.u8("version"),
        borsh.array(borsh.u8(), 7, "padding"),
        borsh.publicKey("lbPair"),
        borsh.array(types.Bin.layout(), 70, "bins"),
    ]);
    constructor(fields) {
        this.index = fields.index;
        this.version = fields.version;
        this.padding = fields.padding;
        this.lbPair = fields.lbPair;
        this.bins = fields.bins.map((item) => new types.Bin({ ...item }));
    }
    static async fetch(c, address, programId = programId_1.PROGRAM_ID) {
        const info = await c.getAccountInfo(address);
        if (info === null) {
            return null;
        }
        if (!info.owner.equals(programId)) {
            throw new Error("account doesn't belong to this program");
        }
        return this.decode(info.data);
    }
    static async fetchMultiple(c, addresses, programId = programId_1.PROGRAM_ID) {
        const infos = await c.getMultipleAccountsInfo(addresses);
        return infos.map((info) => {
            if (info === null) {
                return null;
            }
            if (!info.owner.equals(programId)) {
                throw new Error("account doesn't belong to this program");
            }
            return this.decode(info.data);
        });
    }
    static decode(data) {
        if (!data.slice(0, 8).equals(BinArray.discriminator)) {
            throw new Error("invalid account discriminator");
        }
        const dec = BinArray.layout.decode(data.slice(8));
        return new BinArray({
            index: dec.index,
            version: dec.version,
            padding: dec.padding,
            lbPair: dec.lbPair,
            bins: dec.bins.map((item /* eslint-disable-line @typescript-eslint/no-explicit-any */) => types.Bin.fromDecoded(item)),
        });
    }
    toJSON() {
        return {
            index: this.index.toString(),
            version: this.version,
            padding: this.padding,
            lbPair: this.lbPair.toString(),
            bins: this.bins.map((item) => item.toJSON()),
        };
    }
    static fromJSON(obj) {
        return new BinArray({
            index: new bn_js_1.default(obj.index),
            version: obj.version,
            padding: obj.padding,
            lbPair: new web3_js_1.PublicKey(obj.lbPair),
            bins: obj.bins.map((item) => types.Bin.fromJSON(item)),
        });
    }
}
exports.BinArray = BinArray;
//# sourceMappingURL=BinArray.js.map